/*
 * Copyright 2016 Steve Nickolas.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal with the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 *   1. Redistributions of source code must retain the above copyright notice, 
 *      this list of conditions and the following disclaimers.
 *   2. Redistributions in binary form must reproduce the above copyright 
 *      notice, this list of conditions and the following disclaimers in the 
 *      documentation and/or other materials provided with the distribution.
 *   3. Neither the name of the author, nor the names of its contributors, may 
 *      be used to endorse or promote products derived from this Software 
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 */

#include <stdint.h>
#include <stdio.h>
#include <windows.h>

__declspec(dllexport) uint32_t mod_type=0x524B5053;
__declspec(dllexport) uint8_t *mod_name="Windows Speaker Driver";
__declspec(dllexport) uint8_t *mod_pref="sndwinmm";
__declspec(dllexport) uint8_t *mod_vers="0.01";
__declspec(dllexport) uint8_t *mod_copr="Copyright 2016 Steve Nickolas";

static HWAVEOUT hwaveout;
static WAVEHDR hdr;
static uint8_t sndbuf[157200];
static uint32_t sndbufptr;
static uint8_t sndon, sndhalt;
static uint8_t polarity, amplitude;

static int openwav (void)
{
 WAVEFORMATEX fmt;
 
 fmt.cbSize=0;
 fmt.nChannels=1;
 fmt.nSamplesPerSec=(15720);
 fmt.wBitsPerSample=8;
 fmt.nBlockAlign=(fmt.nChannels*fmt.wBitsPerSample)>>3;
 fmt.nAvgBytesPerSec=fmt.nSamplesPerSec*fmt.nBlockAlign;
 fmt.wFormatTag=WAVE_FORMAT_PCM;
 return waveOutOpen(&hwaveout,WAVE_MAPPER,&fmt,0,0,CALLBACK_NULL);
}

__declspec(dllexport) int sndwinmm_sndinit (void)
{
 if (openwav())
 {
  sndon=0;
  fprintf (stderr, "Could not open sound device\n");
 }
 else
 {
  hdr.dwBufferLength=(157200);
  hdr.dwBytesRecorded=0;
  hdr.dwLoops=0;
  hdr.dwFlags=0;
  hdr.lpData=sndbuf;
  if (waveOutPrepareHeader(hwaveout,&hdr,sizeof(hdr)))
  {
   waveOutClose(hwaveout);
   sndon=0;
   fprintf (stderr, "Could not initialize sound device\n");
  }
  else
  {
   printf ("Sound device initialized successfully\n");
   sndon=1;
  }
 }
 polarity=0;
 amplitude=63;
}

__declspec(dllexport) void sndwinmm_snddeinit (void)
{
 if (sndon)
 {
  waveOutClose(hwaveout);
 }
}

__declspec(dllexport) void sndwinmm_snddo (void)
{
 polarity=!polarity;
}

__declspec(dllexport) void sndwinmm_sndenable (uint8_t enable)
{
 sndhalt=!enable;
}

__declspec(dllexport) void sndwinmm_sndloop (void)
{
 if (!sndon) return;
 if (sndhalt) return;
 
 sndbuf[sndbufptr++]=polarity*amplitude;
 if (sndbufptr>=(157200))
 {
  sndbufptr=0;
 }
 else if (sndbufptr==524)
 {
  waveOutWrite(hwaveout,&hdr,sizeof(hdr));
 }
}

__declspec(dllexport) void sndwinmm_sndconfig (void)
{
}